@@ -60,7 +60,7 @@ main (int argc, char **argv)
 
   srand ((unsigned int)time (NULL));
 
-  err = poclu_get_multiple_devices (&platform, &context, &num_devices,
+  err = poclu_get_multiple_devices (&platform, &context, 0, &num_devices,
                                     &devices, &queues);
 
   CHECK_OPENCL_ERROR_IN ("clCreateContext");
