@@ -22,26 +22,25 @@
 */
 
 #include "pocl_cl.h"
+#include "pocl_shared.h"
 #include "pocl_util.h"
 
-CL_API_ENTRY cl_int CL_API_CALL
-POname(clEnqueueSVMMemFill) (cl_command_queue command_queue,
-                     void *svm_ptr,
-                     const void *pattern,
-                     size_t pattern_size,
-                     size_t size,
-                     cl_uint num_events_in_wait_list,
-                     const cl_event *event_wait_list,
-                     cl_event *event) CL_API_SUFFIX__VERSION_2_0
+cl_int
+pocl_svm_memfill_common (cl_command_type command_type,
+                         cl_command_queue command_queue, void *svm_ptr,
+                         const void *pattern, size_t pattern_size, size_t size,
+                         cl_uint num_events_in_wait_list,
+                         const cl_event *event_wait_list, cl_event *event)
 {
-  unsigned i;
   cl_int errcode;
 
   POCL_RETURN_ERROR_COND ((!IS_CL_OBJECT_VALID (command_queue)),
                           CL_INVALID_COMMAND_QUEUE);
 
+  cl_context context = command_queue->context;
+
   POCL_RETURN_ERROR_ON (
-      (command_queue->context->svm_allocdev == NULL), CL_INVALID_OPERATION,
+      (context->svm_allocdev == NULL), CL_INVALID_OPERATION,
       "None of the devices in this context is SVM-capable\n");
 
   POCL_RETURN_ERROR_COND((svm_ptr == NULL), CL_INVALID_VALUE);
@@ -66,10 +65,14 @@ POname(clEnqueueSVMMemFill) (cl_command_queue command_queue,
   if (errcode != CL_SUCCESS)
     return errcode;
 
+  errcode = pocl_svm_check_pointer (context, svm_ptr, size, NULL);
+  if (errcode != CL_SUCCESS)
+    return errcode;
+
   _cl_command_node *cmd = NULL;
-  errcode = pocl_create_command (&cmd, command_queue, CL_COMMAND_SVM_MEMFILL,
-                                 event, num_events_in_wait_list,
-                                 event_wait_list, 0, NULL, NULL);
+  errcode = pocl_create_command (&cmd, command_queue, command_type, event,
+                                 num_events_in_wait_list, event_wait_list, 0,
+                                 NULL, NULL);
 
   if (errcode != CL_SUCCESS)
     {
@@ -90,4 +93,16 @@ POname(clEnqueueSVMMemFill) (cl_command_queue command_queue,
 
   return CL_SUCCESS;
 }
+
+CL_API_ENTRY cl_int CL_API_CALL
+POname (clEnqueueSVMMemFill) (cl_command_queue command_queue, void *svm_ptr,
+                              const void *pattern, size_t pattern_size,
+                              size_t size, cl_uint num_events_in_wait_list,
+                              const cl_event *event_wait_list,
+                              cl_event *event) CL_API_SUFFIX__VERSION_2_0
+{
+  return pocl_svm_memfill_common (
+      CL_COMMAND_SVM_MEMFILL, command_queue, svm_ptr, pattern, pattern_size,
+      size, num_events_in_wait_list, event_wait_list, event);
+}
 POsym(clEnqueueSVMMemFill)
