@@ -54,15 +54,44 @@ POname(clGetKernelWorkGroupInfo)
       device = kernel->context->devices[0];
     }
 
+  // find device index
+  cl_uint dev_i = CL_UINT_MAX;
+  cl_device_id realdev = pocl_real_dev (device);
+  for (unsigned i = 0; i < kernel->program->num_devices; ++i)
+    {
+      if (kernel->program->devices[i] == realdev)
+        dev_i = i;
+    }
+  POCL_RETURN_ERROR_ON ((dev_i == CL_UINT_MAX), CL_INVALID_KERNEL,
+                        "the kernel was not built for this device\n");
+
+  /************************************************************************/
+
   switch (param_name)
     {
+    case CL_KERNEL_GLOBAL_WORK_SIZE:
+      {
+        /* this parameter is only for custom devices & builtin kernels. */
+        POCL_RETURN_ERROR_ON (
+            (!kernel->meta->builtin_kernel
+             && (device->type != CL_DEVICE_TYPE_CUSTOM)),
+            CL_INVALID_VALUE,
+            "only valid for custom devices or builtin kernels\n");
+        POCL_RETURN_GETINFO (size_t_3, kernel->meta->builtin_max_global_work);
+      }
     case CL_KERNEL_WORK_GROUP_SIZE:
-      return POname(clGetDeviceInfo)
-        (device, CL_DEVICE_MAX_WORK_GROUP_SIZE, param_value_size,
-         param_value, param_value_size_ret);
+      {
+        if (kernel->meta->max_workgroup_size
+            && kernel->meta->max_workgroup_size[dev_i])
+          POCL_RETURN_GETINFO (size_t,
+                               kernel->meta->max_workgroup_size[dev_i]);
+        else // fallback to device's CL_DEVICE_MAX_WORK_GROUP_SIZE
+          return POname (clGetDeviceInfo) (
+              device, CL_DEVICE_MAX_WORK_GROUP_SIZE, param_value_size,
+              param_value, param_value_size_ret);
+      }
     case CL_KERNEL_COMPILE_WORK_GROUP_SIZE:
     {
-        typedef struct { size_t size[3]; } size_t_3;
         POCL_MSG_PRINT_GENERAL (
             "### reqd wg sizes %zu %zu %zu\n", kernel->meta->reqd_wg_size[0],
             kernel->meta->reqd_wg_size[1], kernel->meta->reqd_wg_size[2]);
@@ -70,30 +99,34 @@ POname(clGetKernelWorkGroupInfo)
                              *(size_t_3 *)kernel->meta->reqd_wg_size);
     }
     case CL_KERNEL_PREFERRED_WORK_GROUP_SIZE_MULTIPLE:
-      POCL_RETURN_GETINFO(size_t, device->preferred_wg_size_multiple);
+      {
+        if (kernel->meta->preferred_wg_multiple)
+          POCL_RETURN_GETINFO (size_t,
+                               kernel->meta->preferred_wg_multiple[dev_i]);
+        else // fallback to device's preferred WG size multiple
+          POCL_RETURN_GETINFO (size_t, device->preferred_wg_size_multiple);
+      }
     case CL_KERNEL_LOCAL_MEM_SIZE:
     {
-      size_t local_size = 0, i;
-
-      /* Count the host-allocated locals. */
-      for (i = 0; i < kernel->meta->num_args; ++i)
-        {
-          if (!ARG_IS_LOCAL (kernel->meta->arg_info[i]))
-            continue;
-          local_size += kernel->dyn_arguments[i].size;
-        }
-      /* Count the automatic locals. */
-      for (i = 0; i < kernel->meta->num_locals; ++i)
-        {
-          local_size += kernel->meta->local_sizes[i];
-        }
-      POCL_RETURN_GETINFO(cl_ulong, local_size);
+      if (kernel->meta->local_mem_size)
+        POCL_RETURN_GETINFO (size_t, kernel->meta->local_mem_size[dev_i]);
+      else
+        POCL_RETURN_GETINFO (cl_ulong, 0);
     }
     case CL_KERNEL_PRIVATE_MEM_SIZE:
-      POCL_MSG_WARN ("clGetKernelWorkGroupInfo: CL_KERNEL_PRIVATE_MEM_SIZE "
-                     "implementation is incomplete\n");
-      POCL_RETURN_GETINFO (cl_ulong, 1024);
-
+      {
+        if (kernel->meta->private_mem_size)
+          POCL_RETURN_GETINFO (size_t, kernel->meta->private_mem_size[dev_i]);
+        else
+          POCL_RETURN_GETINFO (cl_ulong, 0);
+      }
+    case CL_KERNEL_SPILL_MEM_SIZE_INTEL:
+      {
+        if (kernel->meta->spill_mem_size)
+          POCL_RETURN_GETINFO (size_t, kernel->meta->spill_mem_size[dev_i]);
+        else
+          POCL_RETURN_GETINFO (cl_ulong, 0);
+      }
     default:
       return CL_INVALID_VALUE;
     }
