@@ -8,7 +8,7 @@
  */ 
 
 #ifdef BUILD_ICD
-CL_API_ENTRY cl_int CL_API_CALL
+POCL_EXPORT CL_API_ENTRY cl_int CL_API_CALL
 POname(clIcdGetPlatformIDsKHR)(cl_uint           num_entries,
                        cl_platform_id *  platforms,
                        cl_uint *         num_platforms) CL_API_SUFFIX__VERSION_1_0
