@@ -51,6 +51,7 @@
 #include "pocl_tracing.h"
 #include "pocl_util.h"
 #include "pocl_export.h"
+#include "pocl_version.h"
 
 #ifdef ENABLE_LLVM
 #include "pocl_llvm.h"
@@ -87,6 +88,11 @@
 #include "vulkan/pocl-vulkan.h"
 #endif
 
+#ifdef BUILD_LEVEL0
+#include "level0/pocl-level0.h"
+#endif
+
+#define MAX_ENV_NAME_LEN 1024
 #define MAX_DEV_NAME_LEN 64
 
 #ifndef PATH_MAX
@@ -149,6 +155,9 @@ static init_device_ops pocl_devices_init_ops[] = {
 #ifdef BUILD_VULKAN
   INIT_DEV (vulkan),
 #endif
+#ifdef BUILD_LEVEL0
+  INIT_DEV (level0),
+#endif
 };
 
 #define POCL_NUM_DEVICE_TYPES (sizeof(pocl_devices_init_ops) / sizeof((pocl_devices_init_ops)[0]))
@@ -178,6 +187,9 @@ char pocl_device_types[POCL_NUM_DEVICE_TYPES][30] = {
 #ifdef BUILD_VULKAN
   "vulkan",
 #endif
+#ifdef BUILD_LEVEL0
+  "level0",
+#endif
 };
 
 static struct pocl_device_ops pocl_device_ops[POCL_NUM_DEVICE_TYPES];
@@ -478,7 +490,7 @@ pocl_init_devices ()
 
   /* first time initialization */
   unsigned i, j, dev_index;
-  char env_name[1024];
+  char env_name[MAX_ENV_NAME_LEN] = { 0 };
   char dev_name[MAX_DEV_NAME_LEN] = { 0 };
 
   /* Set a global debug flag, so we don't have to call pocl_get_bool_option
@@ -611,12 +623,12 @@ pocl_init_devices ()
           dev->driver_version = POCL_VERSION_FULL;
           if (dev->version == NULL)
             dev->version = "OpenCL 2.0 pocl";
-          dev->short_name = strdup (dev->ops->device_name);
 
           /* Check if there are device-specific parameters set in the
              POCL_DEVICEn_PARAMETERS env. */
           POCL_GOTO_ERROR_ON (
-              (snprintf (env_name, 1024, "POCL_%s%d_PARAMETERS", dev_name, j)
+              (snprintf (env_name, MAX_ENV_NAME_LEN,
+                         "POCL_%s%d_PARAMETERS", dev_name, j)
                < 0),
               CL_OUT_OF_HOST_MEMORY, "Unable to generate the env string.");
           errcode = pocl_devices[dev_index].ops->init (
