@@ -28,6 +28,10 @@
 #  error this file must be included through pocl_cl.h, not directly
 #endif
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 POdeclsym(clBuildProgram)
 POdeclsym(clLinkProgram)
 POdeclsym(clCompileProgram)
@@ -98,11 +102,9 @@ POdeclsym(clGetSamplerInfo)
 POdeclsym(clGetSupportedImageFormats)
 POdeclsymICD(clIcdGetPlatformIDsKHR)
 POdeclsym(clReleaseCommandQueue)
-POdeclsym(clReleaseContext)
-POCL_EXPORT
-POdeclsym(clReleaseDevice)
-POCL_EXPORT
-POdeclsym(clReleaseEvent)
+POdeclsymExport(clReleaseContext)
+POdeclsymExport(clReleaseDevice)
+POdeclsymExport(clReleaseEvent)
 POdeclsym(clReleaseKernel)
 POdeclsym(clReleaseMemObject)
 POdeclsym(clReleaseProgram)
@@ -110,8 +112,7 @@ POdeclsym(clReleaseSampler)
 POdeclsym(clRetainCommandQueue)
 POdeclsym(clRetainContext)
 POdeclsym(clRetainDevice)
-POCL_EXPORT
-POdeclsym(clRetainEvent)
+POdeclsymExport(clRetainEvent)
 POdeclsym(clRetainKernel)
 POdeclsym(clRetainMemObject)
 POdeclsym(clRetainProgram)
@@ -158,4 +159,39 @@ POdeclsym(clCreateImageWithProperties)
 POdeclsym(clCloneKernel)
 POdeclsym(clEnqueueSVMMigrateMem)
 
+/* cl_khr_command_buffer */
+POdeclsym(clCreateCommandBufferKHR)
+POdeclsym(clRetainCommandBufferKHR)
+POdeclsym(clReleaseCommandBufferKHR)
+POdeclsym(clFinalizeCommandBufferKHR)
+POdeclsym(clEnqueueCommandBufferKHR)
+POdeclsym(clCommandBarrierWithWaitListKHR)
+POdeclsym(clCommandNDRangeKernelKHR)
+POdeclsym(clCommandCopyBufferKHR)
+POdeclsym(clCommandCopyBufferRectKHR)
+POdeclsym(clCommandCopyBufferToImageKHR)
+POdeclsym(clCommandCopyImageKHR)
+POdeclsym(clCommandCopyImageToBufferKHR)
+POdeclsym(clCommandFillBufferKHR)
+POdeclsym(clCommandFillImageKHR)
+POdeclsym(clGetCommandBufferInfoKHR)
+
+/* cl_intel_unified_shared_memory */
+POdeclsym(clHostMemAllocINTEL)
+POdeclsym(clDeviceMemAllocINTEL)
+POdeclsym(clSharedMemAllocINTEL)
+POdeclsym(clMemFreeINTEL)
+POdeclsym(clMemBlockingFreeINTEL)
+POdeclsym(clGetMemAllocInfoINTEL)
+POdeclsym(clSetKernelArgMemPointerINTEL)
+POdeclsym(clEnqueueMemFillINTEL)
+POdeclsym(clEnqueueMemcpyINTEL)
+POdeclsym(clEnqueueMemAdviseINTEL)
+POdeclsym(clEnqueueMigrateMemINTEL)
+
+
+#ifdef __cplusplus
+}
+#endif
+
 #endif
