@@ -3,6 +3,7 @@
 
    Copyright (c) 2013 Kalle Raiskila
                  2013-2019 Pekka Jääskeläinen
+                 2023 Pekka Jääskeläinen / Intel Finland Oy
 
    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to deal
@@ -29,11 +30,15 @@
 #include "pocl_cache.h"
 #include "pocl_file_util.h"
 #include "pocl_llvm_api.h"
+#include "pocl_spir.h"
+#include "pocl_util.h"
 
-#include <string>
-#include <map>
-#include <vector>
 #include <iostream>
+#include <map>
+#include <regex>
+#include <string>
+#include <thread>
+#include <vector>
 
 #include "CompilerWarnings.h"
 IGNORE_COMPILER_WARNING("-Wunused-parameter")
@@ -50,8 +55,9 @@ IGNORE_COMPILER_WARNING("-Wunused-parameter")
 #include <llvm/ADT/Triple.h>
 #include <llvm/ADT/StringRef.h>
 
-#include <llvm/IR/Module.h>
 #include <llvm/IR/Function.h>
+#include <llvm/IR/Module.h>
+#include <llvm/IR/Verifier.h>
 
 #include <llvm/Target/TargetOptions.h>
 #include <llvm/Target/TargetMachine.h>
@@ -74,12 +80,8 @@ IGNORE_COMPILER_WARNING("-Wunused-parameter")
 // each work-group IR function generation. Requires LLVM > 7.
 // #define DUMP_LLVM_PASS_TIMINGS
 
-#ifndef LLVM_OLDER_THAN_10_0
 #include <llvm/IR/PassTimingInfo.h>
 #define CODEGEN_FILE_TYPE_NS llvm
-#else
-#define CODEGEN_FILE_TYPE_NS TargetMachine
-#endif
 
 using namespace llvm;
 
@@ -210,17 +212,28 @@ static PassManager &kernel_compiler_passes(cl_device_id device) {
      -phistoallocas before -workitemloops as otherwise it cannot inject context
      restore code (PHIs need to be at the beginning of the BB and so one cannot
      context restore them with non-PHI code if the value is needed in another
-     PHI). */
+     PHI).
+
+     -automatic-locals after inline and always-inline; if we have a kernel
+     that calls a non-kernel, and the non-kernel uses an automatic local
+     (= GlobalVariable in LLVM), the 'automatic-locals' will skip processing
+     of the non-kernel function, and the kernel function appears to it as not
+     having any locals. Therefore the local variable remains a GV instead of
+     being transformed into a kernel argument. This can lead to surprising
+     result, as the final object ELF will contain a static variable, so the
+     program will work with single-threaded execution, but multiple CPU
+     threads will overwrite the static variable and produce garbage results.
+  */
 
   std::vector<std::string> passes;
   passes.push_back("inline-kernels");
   passes.push_back("remove-optnone");
   passes.push_back("optimize-wi-func-calls");
   passes.push_back("handle-samplers");
+  passes.push_back("infer-address-spaces");
   passes.push_back("workitem-handler-chooser");
   passes.push_back("mem2reg");
   passes.push_back("domtree");
-  passes.push_back("automatic-locals");
 
   if (SPMDDevice) {
     passes.push_back("flatten-inline-all");
@@ -232,6 +245,9 @@ static PassManager &kernel_compiler_passes(cl_device_id device) {
     passes.push_back("inline");
   }
 
+  // this must be done AFTER inlining, see note above
+  passes.push_back("automatic-locals");
+
   // It should be now safe to run -O3 over the single work-item kernel
   // as the barrier has the attributes preventing illegal motions and
   // duplication. Let's do it to clean up the code for later passes.
@@ -254,6 +270,10 @@ static PassManager &kernel_compiler_passes(cl_device_id device) {
     passes.push_back("wi-aa");
     passes.push_back("workitemrepl");
     //passes.push_back("print-module");
+    passes.push_back("subcfgformation");
+    // subcfgformation before workitemloops, as wiloops creates the loops for
+    // kernels without barriers, but after the transformation the kernel looks
+    // like it has barriers, so subcfg would do its thing.
     passes.push_back("workitemloops");
     // Remove the (pseudo) barriers.   They have no use anymore due to the
     // work-item loop control taking care of them.
@@ -306,15 +326,16 @@ static PassManager &kernel_compiler_passes(cl_device_id device) {
       // to get the vectorizers initialized properly. Assume SPMD
       // devices do not want to vectorize intra work-item at this
       // stage.
-      if (currentWgMethod == "loopvec" && !SPMDDevice) {
+      if ((CurrentWgMethod == "loopvec" || CurrentWgMethod == "cbs") &&
+          !SPMDDevice) {
         Builder.LoopVectorize = true;
         Builder.SLPVectorize = true;
       } else {
         Builder.LoopVectorize = false;
         Builder.SLPVectorize = false;
       }
-      Builder.VerifyInput = true;
-      Builder.VerifyOutput = true;
+      Builder.VerifyInput = LLVM_VERIFY_MODULE_DEFAULT > 0;
+      Builder.VerifyOutput = LLVM_VERIFY_MODULE_DEFAULT > 0;
       Builder.populateModulePassManager(*Passes);
       continue;
     }
@@ -351,9 +372,252 @@ void pocl_destroy_llvm_module(void *modp, cl_context ctx) {
   }
 }
 
-// The global variables used to control the WG function generation's
-// specialization parameteres. Defined in lib/llvmopencl/WorkitemHandler.cc.
 namespace pocl {
+class ProgramWithContext {
+
+  llvm::LLVMContext LLVMCtx;
+  std::unique_ptr<llvm::Module> ProgramBC;
+  std::unique_ptr<llvm::Module> ProgramGVarsNonKernelsBC;
+  std::mutex Lock;
+  unsigned Num = 0;
+
+public:
+
+  bool init(const char *ProgramBcBytes,
+            size_t ProgramBcSize,
+            char* LinkinOutputBCPath) {
+    Num = 0;
+    llvm::Module *P = parseModuleIRMem(ProgramBcBytes, ProgramBcSize, &LLVMCtx);
+    if (P == nullptr)
+      return false;
+    ProgramBC.reset(P);
+
+    ProgramGVarsNonKernelsBC.reset(
+        new llvm::Module(llvm::StringRef("program_gvars.bc"), LLVMCtx));
+
+    ProgramGVarsNonKernelsBC->setTargetTriple(ProgramBC->getTargetTriple());
+    ProgramGVarsNonKernelsBC->setDataLayout(ProgramBC->getDataLayout());
+
+    if (!moveProgramScopeVarsOutOfProgramBc(&LLVMCtx, ProgramBC.get(),
+                                            ProgramGVarsNonKernelsBC.get(),
+                                            SPIR_ADDRESS_SPACE_LOCAL))
+      return false;
+
+    pocl_cache_tempname(LinkinOutputBCPath, ".bc", NULL);
+    int r = pocl_write_module(ProgramGVarsNonKernelsBC.get(),
+                              LinkinOutputBCPath, 0);
+    if (r != 0) {
+      POCL_MSG_ERR("ProgramWithContext->init: failed to write module\n");
+      return false;
+    }
+
+    if (pocl_get_bool_option("POCL_LLVM_VERIFY", LLVM_VERIFY_MODULE_DEFAULT)) {
+      std::string ErrorLog;
+      llvm::raw_string_ostream Errs(ErrorLog);
+      if (llvm::verifyModule(*ProgramGVarsNonKernelsBC.get(), &Errs)) {
+        POCL_MSG_ERR("Failed to verify Program GVars Module:\n%s\n",
+                     ErrorLog.c_str());
+        return false;
+      }
+    }
+
+    return true;
+  }
+
+  bool getBitcodeForKernel(const char* KernelName,
+                           char* OutputPath,
+                           std::string *BuildLog) {
+    std::lock_guard<std::mutex> LockGuard(Lock);
+
+    // Create an empty Module and copy only the kernel+callgraph from
+    // program.bc.
+    std::unique_ptr<llvm::Module> KernelBC(
+        new llvm::Module(llvm::StringRef("parallel_bc"), LLVMCtx));
+
+    KernelBC->setTargetTriple(ProgramBC->getTargetTriple());
+    KernelBC->setDataLayout(ProgramBC->getDataLayout());
+
+    copyKernelFromBitcode(KernelName, KernelBC.get(), ProgramBC.get(), nullptr);
+
+    if (pocl_get_bool_option("POCL_LLVM_VERIFY", LLVM_VERIFY_MODULE_DEFAULT)) {
+      llvm::raw_string_ostream Errs(*BuildLog);
+      if (llvm::verifyModule(*KernelBC.get(), &Errs)) {
+        POCL_MSG_ERR("Failed to verify Kernel Module:\n%s\n",
+                     BuildLog->c_str());
+        BuildLog->append("Failed to verify Kernel Module\n");
+        return false;
+      }
+    }
+
+    pocl_cache_tempname(OutputPath, ".bc", NULL);
+    int r = pocl_write_module(KernelBC.get(), OutputPath, 0);
+    if (r != 0) {
+      POCL_MSG_ERR("getBitcodeForKernel: failed to write module\n");
+      BuildLog->append("getBitcodeForKernel: failed to write module\n");
+      return false;
+    }
+    return true;
+  }
+};
+
+static int convertBitcodeToSpv(char* TempBitcodePath,
+                               std::string *BuildLog,
+                               char **SpirvContent,
+                               uint64_t *SpirvSize) {
+
+  char TempSpirvPath[POCL_MAX_PATHNAME_LENGTH];
+
+// max bytes in output of 'llvm-spirv'
+#define MAX_OUTPUT_BYTES 65536
+
+//   --spirv-ext=<+SPV_extenstion1_name,-SPV_extension2_name>
+//   Specify list of allowed/disallowed extensions
+#define ALLOW_EXTS                                                             \
+  "--spirv-ext=+SPV_INTEL_subgroups,+SPV_INTEL_usm_storage_classes,+SPV_"      \
+  "INTEL_arbitrary_precision_integers,+SPV_INTEL_arbitrary_precision_fixed_"   \
+  "point,+SPV_INTEL_arbitrary_precision_floating_point,+SPV_INTEL_kernel_"     \
+  "attributes"
+  /*
+  possibly useful:
+    "+SPV_INTEL_unstructured_loop_controls,"
+    "+SPV_INTEL_blocking_pipes,"
+    "+SPV_INTEL_function_pointers,"
+    "+SPV_INTEL_io_pipes,"
+    "+SPV_INTEL_inline_assembly,"
+    "+SPV_INTEL_optimization_hints,"
+    "+SPV_INTEL_float_controls2,"
+    "+SPV_INTEL_vector_compute,"
+    "+SPV_INTEL_fast_composite,"
+    "+SPV_INTEL_variable_length_array,"
+    "+SPV_INTEL_fp_fast_math_mode,"
+    "+SPV_INTEL_long_constant_composite,"
+    "+SPV_INTEL_memory_access_aliasing,"
+    "+SPV_INTEL_runtime_aligned,"
+    "+SPV_INTEL_arithmetic_fence,"
+    "+SPV_INTEL_bfloat16_conversion,"
+    "+SPV_INTEL_global_variable_decorations,"
+    "+SPV_INTEL_non_constant_addrspace_printf,"
+    "+SPV_INTEL_hw_thread_queries,"
+    "+SPV_INTEL_complex_float_mul_div,"
+    "+SPV_INTEL_split_barrier,"
+    "+SPV_INTEL_masked_gather_scatter"
+
+  probably not useful:
+    "+SPV_INTEL_media_block_io,+SPV_INTEL_device_side_avc_motion_estimation,"
+    "+SPV_INTEL_fpga_loop_controls,+SPV_INTEL_fpga_memory_attributes,"
+    "+SPV_INTEL_fpga_memory_accesses,"
+    "+SPV_INTEL_fpga_reg,+SPV_INTEL_fpga_buffer_location,"
+    "+SPV_INTEL_fpga_cluster_attributes,"
+    "+SPV_INTEL_loop_fuse,"
+    "+SPV_INTEL_optnone," // this one causes crash
+    "+SPV_INTEL_fpga_dsp_control,"
+    "+SPV_INTEL_fpga_invocation_pipelining_attributes,"
+    "+SPV_INTEL_token_type,"
+    "+SPV_INTEL_debug_module,"
+    "+SPV_INTEL_joint_matrix,"
+  */
+  pocl_cache_tempname(TempSpirvPath, ".spirv", NULL);
+  char LLVMspirv[] = LLVM_SPIRV;
+  char AllowedExtOption[] = ALLOW_EXTS;
+  // TODO ze_device_module_properties_t.spirvVersionSupported
+  char MaxSPIRVOption[] = "--spirv-max-version=1.2";
+#ifdef LLVM_OPAQUE_POINTERS
+  char OpaquePtrsOption[] = "--opaque-pointers";
+#endif
+  char OutputOption[] = { '-', 'o', 0 };
+  char *CmdArgs[] = { LLVMspirv, AllowedExtOption,
+#ifdef LLVM_OPAQUE_POINTERS
+                      OpaquePtrsOption,
+#endif
+                      MaxSPIRVOption, OutputOption,
+                      TempSpirvPath, TempBitcodePath, NULL };
+  char CapturedOutput[MAX_OUTPUT_BYTES];
+  size_t CapturedBytes = MAX_OUTPUT_BYTES;
+
+  int r =
+      pocl_run_command_capture_output(CapturedOutput, &CapturedBytes, CmdArgs);
+  if (r != 0) {
+    BuildLog->append("llvm-spirv failed with output:\n");
+    std::string Captured(CapturedOutput, CapturedBytes);
+    BuildLog->append(Captured);
+    return -1;
+  }
+
+  r = pocl_read_file(TempSpirvPath, SpirvContent, SpirvSize);
+  if (r != 0) {
+    BuildLog->append("failed to read output file from llvm-spirv\n");
+    return -1;
+  }
+
+  if (pocl_get_bool_option("POCL_LEAVE_KERNEL_COMPILER_TEMP_FILES", 0) == 0) {
+    pocl_remove(TempBitcodePath);
+    pocl_remove(TempSpirvPath);
+  } else {
+    POCL_MSG_PRINT_GENERAL("LLVM SPIR-V conversion tempfiles: %s -> %s",
+                           TempBitcodePath, TempSpirvPath);
+  }
+  return 0;
+}
+
+} // namespace pocl
+
+void *pocl_llvm_create_context_for_program(const char *ProgramBcBytes,
+                                           size_t ProgramBcSize,
+                                           char **LinkinSpirvContent,
+                                           uint64_t *LinkinSpirvSize) {
+  assert(ProgramBcBytes);
+  assert(ProgramBcSize > 0);
+
+  char TempBitcodePath[POCL_MAX_PATHNAME_LENGTH];
+
+  pocl::ProgramWithContext *P = new pocl::ProgramWithContext;
+  // parse the program's bytes into a llvm::Module
+  if (P == nullptr ||
+      !P->init(ProgramBcBytes, ProgramBcSize, TempBitcodePath)) {
+    POCL_MSG_ERR("failed to create program for context");
+    return nullptr;
+  }
+
+  std::string BuildLog;
+  if (pocl::convertBitcodeToSpv(TempBitcodePath, &BuildLog,
+                                LinkinSpirvContent, LinkinSpirvSize) != 0) {
+    POCL_MSG_ERR("failed to create program for context, log:%s\n",
+                 BuildLog.c_str());
+    return nullptr;
+  }
+
+  return (void *)P;
+}
+
+void pocl_llvm_release_context_for_program(void *ProgCtx) {
+  if (ProgCtx == nullptr)
+    return;
+  pocl::ProgramWithContext *P = (pocl::ProgramWithContext *)ProgCtx;
+  delete P;
+}
+
+// extract SPIRV of a single Kernel from a program
+int pocl_llvm_extract_kernel_spirv(void* ProgCtx,
+                                   const char* KernelName,
+                                   void* BuildLogStr,
+                                   char **SpirvContent,
+                                   uint64_t *SpirvSize) {
+
+  POCL_MEASURE_START(extractKernel);
+
+  std::string *BuildLog = (std::string *)BuildLogStr;
+
+  char TempBitcodePath[POCL_MAX_PATHNAME_LENGTH];
+  pocl::ProgramWithContext *P = (pocl::ProgramWithContext *)ProgCtx;
+  if (!P->getBitcodeForKernel(KernelName, TempBitcodePath, BuildLog)) {
+    return -1;
+  }
+
+  int r = pocl::convertBitcodeToSpv(TempBitcodePath, BuildLog,
+                                    SpirvContent, SpirvSize);
+
+  POCL_MEASURE_FINISH(extractKernel);
+  return r;
 }
 
 int pocl_llvm_generate_workgroup_function_nowrite(
@@ -363,28 +627,28 @@ int pocl_llvm_generate_workgroup_function_nowrite(
   _cl_command_run *RunCommand = &Command->command.run;
   cl_program Program = Kernel->program;
   cl_context ctx = Program->context;
-  PoclLLVMContextData *llvm_ctx = (PoclLLVMContextData *)ctx->llvm_context_data;
-  PoclCompilerMutexGuard lockHolder(&llvm_ctx->Lock);
+  PoclLLVMContextData *PoCLLLVMContext =
+      (PoclLLVMContextData *)ctx->llvm_context_data;
+  PoclCompilerMutexGuard lockHolder(&PoCLLLVMContext->Lock);
+  llvm::LLVMContext *LLVMContext = PoCLLLVMContext->Context;
 
 #ifdef DEBUG_POCL_LLVM_API
   printf("### calling the kernel compiler for kernel %s local_x %zu "
          "local_y %zu local_z %zu parallel_filename: %s\n",
          kernel->name, local_x, local_y, local_z, parallel_bc_path);
 #endif
-  assert(Program->data[DeviceI] != nullptr);
-
-  llvm::Module *ProgramBC = (llvm::Module *)Program->data[DeviceI];
+  llvm::Module *ProgramBC = (llvm::Module *)Program->llvm_irs[DeviceI];
 
   // Create an empty Module and copy only the kernel+callgraph from
   // program.bc.
   llvm::Module *ParallelBC =
-      new llvm::Module(StringRef("parallel_bc"), *llvm_ctx->Context);
+      new llvm::Module(StringRef("parallel_bc"), *LLVMContext);
 
   ParallelBC->setTargetTriple(ProgramBC->getTargetTriple());
   ParallelBC->setDataLayout(ProgramBC->getDataLayout());
 
   copyKernelFromBitcode(Kernel->name, ParallelBC, ProgramBC,
-                        Device->global_as_id, Device->device_aux_functions);
+                        Device->device_aux_functions);
 
   // Set to true to generate a global offset 0 specialized WG function.
   bool WGAssumeZeroGlobalOffset;
@@ -489,9 +753,28 @@ int pocl_llvm_generate_workgroup_function_nowrite(
     std::cout << getDiagString(ctx);
   }
 
+  std::string FinalizerCommand =
+      pocl_get_string_option("POCL_BITCODE_FINALIZER", "");
+  if (FinalizerCommand != "") {
+    // Run a user-defined command on the final bitcode.
+    char TempParallelBCFileName[POCL_MAX_PATHNAME_LENGTH];
+    int FD = -1, Err = 0;
+
+    Err = pocl_mk_tempname(TempParallelBCFileName, "/tmp/pocl-parallel", ".bc",
+                           &FD);
+    pocl_write_module((char *)ParallelBC, TempParallelBCFileName, 0);
+
+    std::string Command = std::regex_replace(
+        FinalizerCommand, std::regex(R"(%\(bc\))"), TempParallelBCFileName);
+    system(Command.c_str());
+
+    delete ParallelBC;
+    ParallelBC = parseModuleIR(TempParallelBCFileName, LLVMContext);
+  }
+
   assert(Output != NULL);
   *Output = (void *)ParallelBC;
-  ++llvm_ctx->number_of_IRs;
+  ++PoCLLLVMContext->number_of_IRs;
   return 0;
 }
 
@@ -502,14 +785,14 @@ int pocl_llvm_generate_workgroup_function(unsigned DeviceI, cl_device_id Device,
   cl_context ctx = Kernel->context;
   void *Module = NULL;
 
-  char ParallelBCPath[POCL_FILENAME_LENGTH];
+  char ParallelBCPath[POCL_MAX_PATHNAME_LENGTH];
   pocl_cache_work_group_function_path(ParallelBCPath, Kernel->program, DeviceI,
                                       Kernel, Command, Specialize);
 
   if (pocl_exists(ParallelBCPath))
     return CL_SUCCESS;
 
-  char FinalBinaryPath[POCL_FILENAME_LENGTH];
+  char FinalBinaryPath[POCL_MAX_PATHNAME_LENGTH];
   pocl_cache_final_binary_path(FinalBinaryPath, Kernel->program, DeviceI,
                                Kernel, Command, Specialize);
 
@@ -534,27 +817,31 @@ int pocl_llvm_generate_workgroup_function(unsigned DeviceI, cl_device_id Device,
   return Error;
 }
 
-/* Reads LLVM IR module from program->binaries[i], if program->data[device_i] is
+/* Reads LLVM IR module from program->binaries[i], if prog_data->llvm_ir is
  * NULL */
 int pocl_llvm_read_program_llvm_irs(cl_program program, unsigned device_i,
                                     const char *program_bc_path) {
   cl_context ctx = program->context;
   PoclLLVMContextData *llvm_ctx = (PoclLLVMContextData *)ctx->llvm_context_data;
   PoclCompilerMutexGuard lockHolder(&llvm_ctx->Lock);
+  cl_device_id dev = program->devices[device_i];
 
-  if (program->data[device_i] != nullptr)
+  if (program->llvm_irs[device_i] != nullptr)
     return CL_SUCCESS;
 
+  llvm::Module *M;
   if (program->binaries[device_i])
-    program->data[device_i] =
-        parseModuleIRMem((char *)program->binaries[device_i],
+    M = parseModuleIRMem((char *)program->binaries[device_i],
                          program->binary_sizes[device_i], llvm_ctx->Context);
   else {
     // TODO
     assert(program_bc_path);
-    program->data[device_i] = parseModuleIR(program_bc_path, llvm_ctx->Context);
+    M = parseModuleIR(program_bc_path, llvm_ctx->Context);
   }
-  assert(program->data[device_i]);
+  assert(M);
+  program->llvm_irs[device_i] = M;
+  if (dev->program_scope_variables_pass)
+    parseModuleGVarSize(program, device_i, M);
   ++llvm_ctx->number_of_IRs;
   return CL_SUCCESS;
 }
@@ -563,61 +850,15 @@ void pocl_llvm_free_llvm_irs(cl_program program, unsigned device_i) {
   cl_context ctx = program->context;
   PoclLLVMContextData *llvm_ctx = (PoclLLVMContextData *)ctx->llvm_context_data;
   PoclCompilerMutexGuard lockHolder(&llvm_ctx->Lock);
-  if (program->data[device_i]) {
-    llvm::Module *mod = (llvm::Module *)program->data[device_i];
+
+  if (program->llvm_irs[device_i]) {
+    llvm::Module *mod = (llvm::Module *)program->llvm_irs[device_i];
     delete mod;
     --llvm_ctx->number_of_IRs;
-    program->data[device_i] = nullptr;
+    program->llvm_irs[device_i] = nullptr;
   }
 }
 
-/* writes fresh program.bc and program->binaries[i] from LLVM IR module. */
-int pocl_llvm_update_binaries(cl_program program, cl_uint device_i) {
-
-  cl_context ctx = program->context;
-  PoclLLVMContextData *llvm_ctx = (PoclLLVMContextData *)ctx->llvm_context_data;
-  PoclCompilerMutexGuard lockHolder(&llvm_ctx->Lock);
-
-  char program_bc_path[POCL_FILENAME_LENGTH];
-  int error;
-
-  // Dump the LLVM IR Modules to memory buffers.
-  assert(program->data != NULL);
-#ifdef DEBUG_POCL_LLVM_API
-  printf("### refreshing the binaries of the program %p\n", program);
-#endif
-
-  cl_uint i = device_i;
-  // LLVM IR
-  assert(program->data[i] != NULL);
-
-  POCL_MEM_FREE(program->binaries[i]);
-
-  pocl_cache_program_bc_path(program_bc_path, program, i);
-  error =
-      pocl_write_module((llvm::Module *)program->data[i], program_bc_path, 1);
-  assert(error == 0);
-  if (error) {
-    POCL_MSG_ERR("pocl_write_module(%s) failed!\n", program_bc_path);
-    return error;
-  }
-
-  std::string content;
-  writeModuleIRtoString((llvm::Module *)program->data[i], content);
-
-  size_t n = content.size();
-  if (n < program->binary_sizes[i])
-    POCL_ABORT("binary size doesn't match the expected value\n");
-
-  program->binaries[i] = (unsigned char *)malloc(n);
-  std::memcpy(program->binaries[i], content.c_str(), n);
-
-#ifdef DEBUG_POCL_LLVM_API
-  printf("### binary for device %zi was of size %zu\n", i,
-         program->binary_sizes[i]);
-#endif
-  return CL_SUCCESS;
-}
 
 static void initPassManagerForCodeGen(PassManager& PM, cl_device_id Device) {
 
@@ -656,10 +897,7 @@ int pocl_llvm_codegen(cl_device_id Device, cl_program program, void *Modp,
   llvm::raw_svector_ostream SOS(Data);
   bool cannotEmitFile;
 
-  cannotEmitFile = Target->addPassesToEmitFile(PMObj, SOS,
-#ifndef LLVM_OLDER_THAN_7_0
-                                  nullptr,
-#endif
+  cannotEmitFile = Target->addPassesToEmitFile(PMObj, SOS, nullptr,
                                   CODEGEN_FILE_TYPE_NS::CGFT_ObjectFile);
 
   LLVMGeneratesObjectFiles = !cannotEmitFile;
@@ -691,11 +929,7 @@ int pocl_llvm_codegen(cl_device_id Device, cl_program program, void *Modp,
   // Have to emit the text first and then call the assembler from the command line
   // to produce the binary.
 
-
-  if (Target->addPassesToEmitFile(PMAsm, SOS,
-#ifndef LLVM_OLDER_THAN_7_0
-                                  nullptr,
-#endif
+  if (Target->addPassesToEmitFile(PMAsm, SOS, nullptr,
                                   CODEGEN_FILE_TYPE_NS::CGFT_AssemblyFile)) {
     POCL_ABORT("The target supports neither obj nor asm emission!");
   }
@@ -715,8 +949,8 @@ int pocl_llvm_codegen(cl_device_id Device, cl_program program, void *Modp,
   // Next call the target's assembler via the Toolchain API indirectly through
   // the Driver API.
 
-  char AsmFileName[POCL_FILENAME_LENGTH];
-  char ObjFileName[POCL_FILENAME_LENGTH];
+  char AsmFileName[POCL_MAX_PATHNAME_LENGTH];
+  char ObjFileName[POCL_MAX_PATHNAME_LENGTH];
 
   std::string AsmStr = SOS.str().str();
   pocl_write_tempfile(AsmFileName, "/tmp/pocl-asm", ".s", AsmStr.c_str(),
