@@ -75,8 +75,9 @@ int context_set_properties (cl_context context,
                             const cl_context_properties *properties);
 
 cl_mem pocl_create_memobject (cl_context context, cl_mem_flags flags,
-                              size_t size, cl_mem_object_type type, int *device_image_support, void *host_ptr,
-                              cl_int *errcode_ret);
+                              size_t size, cl_mem_object_type type,
+                              int *device_image_support, void *host_ptr,
+                              int host_ptr_is_svm, cl_int *errcode_ret);
 
 cl_int pocl_kernel_copy_args (cl_kernel kernel, _cl_command_run *command);
 
@@ -149,6 +150,39 @@ cl_int pocl_fill_image_common (
     cl_sync_point_khr *sync_point, cl_mutable_command_khr *mutable_handle,
     _cl_command_node **cmd);
 
+cl_int pocl_svm_memcpy_common (cl_command_type command_type,
+                               cl_command_queue command_queue,
+                               cl_bool blocking, void *dst_ptr,
+                               const void *src_ptr, size_t size,
+                               cl_uint num_events_in_wait_list,
+                               const cl_event *event_wait_list,
+                               cl_event *event);
+
+cl_int pocl_svm_memfill_common (cl_command_type command_type,
+                                cl_command_queue command_queue,
+                                void *svm_ptr,
+                                const void *pattern,
+                                size_t pattern_size,
+                                size_t size,
+                                cl_uint num_events_in_wait_list,
+                                const cl_event *event_wait_list,
+                                cl_event *event);
+
+cl_int pocl_svm_migrate_mem_common (cl_command_type command_type,
+                                    cl_command_queue command_queue,
+                                    cl_uint num_svm_pointers,
+                                    const void **svm_pointers,
+                                    const size_t *sizes,
+                                    cl_mem_migration_flags flags,
+                                    cl_uint num_events_in_wait_list,
+                                    const cl_event *event_wait_list,
+                                    cl_event *event);
+
+int
+pocl_set_kernel_arg_pointer(cl_kernel kernel,
+                            cl_uint arg_index,
+                            const void *arg_value);
+
 #ifdef __GNUC__
 #pragma GCC visibility pop
 #endif
