@@ -604,10 +604,8 @@ can_run_command (cl_device_id dev, size_t num_objs, cl_mem *objs)
       assert (dev->ops->alloc_mem_obj);
       errcode = dev->ops->alloc_mem_obj (dev, objs[i], NULL);
       if (errcode != CL_SUCCESS) {
-        POCL_UNLOCK_OBJ (objs[i]);
         POCL_MSG_ERR("Failed to allocate %zx bytes on device %s\n",
                      objs[i]->size, dev->short_name);
-        return CL_OUT_OF_RESOURCES;
       }
 
       POCL_UNLOCK_OBJ (objs[i]);
@@ -1727,6 +1725,10 @@ pocl_setup_context (cl_context context)
                      < dev->svm_allocation_priority)
             {
               context->svm_allocdev = dev;
+              if (dev->ops->usm_alloc && dev->ops->usm_free)
+              {
+                  context->usm_allocdev = dev;
+              }
             }
         }
 
@@ -2646,3 +2648,83 @@ pocl_fill_aligned_buf_with_pattern (void *__restrict__ ptr, size_t offset,
     }
   return 0;
 }
+
+void
+pocl_free_kernel_metadata (cl_program program, unsigned kernel_i)
+{
+  pocl_kernel_metadata_t *meta = &program->kernel_meta[kernel_i];
+  unsigned j;
+  POCL_MEM_FREE (meta->attributes);
+  POCL_MEM_FREE (meta->name);
+  for (j = 0; j < meta->num_args; ++j)
+    {
+      POCL_MEM_FREE (meta->arg_info[j].name);
+      POCL_MEM_FREE (meta->arg_info[j].type_name);
+    }
+  POCL_MEM_FREE (meta->max_subgroups);
+  POCL_MEM_FREE (meta->compile_subgroups);
+  POCL_MEM_FREE (meta->max_workgroup_size);
+  POCL_MEM_FREE (meta->preferred_wg_multiple);
+  POCL_MEM_FREE (meta->local_mem_size);
+  POCL_MEM_FREE (meta->private_mem_size);
+  POCL_MEM_FREE (meta->spill_mem_size);
+  POCL_MEM_FREE (meta->arg_info);
+  if (meta->data != NULL)
+    for (j = 0; j < program->num_devices; ++j)
+      if (meta->data[j] != NULL)
+        {
+          POCL_MSG_WARN ("kernel metadata not freed\n");
+          meta->data[j] = NULL; // TODO free data in driver callback
+        }
+  POCL_MEM_FREE (meta->data);
+  POCL_MEM_FREE (meta->local_sizes);
+  POCL_MEM_FREE (meta->build_hash);
+}
+
+int
+pocl_svm_check_pointer (cl_context context, const void *svm_ptr, size_t size,
+                        size_t *buffer_size)
+{
+
+  /* TODO we need a better data structure than linked list,
+   * right now it does a linear scan of all SVM allocations. */
+  POCL_LOCK_OBJ (context);
+  pocl_svm_ptr *found = NULL, *item = NULL;
+  char *svm_alloc_end = NULL;
+  char *svm_alloc_start = NULL;
+  DL_FOREACH (context->svm_ptrs, item)
+  {
+    svm_alloc_start = (char *)item->svm_ptr;
+    svm_alloc_end = svm_alloc_start + item->size;
+    if (((char *)svm_ptr >= svm_alloc_start)
+        && ((char *)svm_ptr < svm_alloc_end))
+      {
+        found = item;
+        break;
+      }
+  }
+  POCL_UNLOCK_OBJ (context);
+
+  /* if the device does not support system allocation,
+   * then the pointer must be found in the context's SVM alloc list */
+  if (found == NULL
+      && (context->svm_allocdev->svm_caps & (CL_DEVICE_SVM_FINE_GRAIN_SYSTEM))
+             == 0)
+    {
+      POCL_MSG_ERR (
+          "Can't find the pointer %p in list of allocated SVM pointers\n",
+            svm_ptr);
+      return CL_INVALID_OPERATION;
+    }
+
+  if (found != NULL && (((char *)svm_ptr + size) > svm_alloc_end))
+    {
+      POCL_MSG_ERR ("The pointer+size exceeds the size of the allocation\n");
+      return CL_INVALID_OPERATION;
+    }
+
+  if (found != NULL && buffer_size != NULL)
+    *buffer_size = item->size;
+
+  return CL_SUCCESS;
+}
