@@ -130,24 +130,6 @@ AutomaticLocals::runOnModule(Module &M)
   return changed;
 }
 
-// Recursively descend a Value's users and convert any constant expressions into
-// regular instructions.
-static void breakConstantExpressions(llvm::Value *Val, llvm::Function *Func) {
-  std::vector<llvm::Value *> Users(Val->user_begin(), Val->user_end());
-  for (auto *U : Users) {
-    if (auto *CE = llvm::dyn_cast<llvm::ConstantExpr>(U)) {
-      // First, make sure no users of this constant expression are themselves
-      // constant expressions.
-      breakConstantExpressions(U, Func);
-
-      // Convert this constant expression to an instruction.
-      llvm::Instruction *I = CE->getAsInstruction();
-      I->insertBefore(&*Func->begin()->begin());
-      CE->replaceAllUsesWith(I);
-      CE->destroyConstant();
-    }
-  }
-}
 
 Function *
 AutomaticLocals::processAutomaticLocals(Function *F) {
@@ -163,9 +145,8 @@ AutomaticLocals::processAutomaticLocals(Function *F) {
 
   for (Module::global_iterator i = M->global_begin(),
          e = M->global_end(); i != e; ++i) {
-    std::string FuncName = "";
-    FuncName = F->getName().str();
-    if (isAutomaticLocal(FuncName, *i)) {
+
+    if (isAutomaticLocal(F, *i)) {
       Locals.push_back(&*i);
 
       // Add the parameters to the end of the function parameter list.
