@@ -36,10 +36,11 @@ IGNORE_COMPILER_WARNING("-Wunused-parameter")
 
 #include "llvm/Analysis/LoopInfo.h"
 
-#include "ImplicitLoopBarriers.h"
 #include "Barrier.h"
-#include "Workgroup.h"
+#include "ImplicitLoopBarriers.h"
 #include "VariableUniformityAnalysis.h"
+#include "Workgroup.h"
+#include "WorkitemHandlerChooser.h"
 
 #include "pocl_runtime_config.h"
 
@@ -61,12 +62,18 @@ void ImplicitLoopBarriers::getAnalysisUsage(AnalysisUsage &AU) const {
   AU.addPreserved<DominatorTreeWrapperPass>();
   AU.addRequired<VariableUniformityAnalysis>();
   AU.addPreserved<VariableUniformityAnalysis>();
+  AU.addRequired<WorkitemHandlerChooser>();
+  AU.addPreserved<WorkitemHandlerChooser>();
 }
 
 bool ImplicitLoopBarriers::runOnLoop(Loop *L, LPPassManager &LPM) {
   if (!Workgroup::isKernelToProcess(*L->getHeader()->getParent()))
     return false;
 
+  if (getAnalysis<WorkitemHandlerChooser>().chosenHandler() ==
+      WorkitemHandlerChooser::POCL_WIH_CBS)
+    return false;
+
   if (!pocl_get_bool_option("POCL_FORCE_PARALLEL_OUTER_LOOP", 0) &&
       !Workgroup::hasWorkgroupBarriers(*L->getHeader()->getParent())) {
 #ifdef DEBUG_ILOOP_BARRIERS
