@@ -201,12 +201,18 @@ ParallelRegion::chainAfter(ParallelRegion *region)
 #endif
 
   BasicBlock *successor = t->getSuccessor(0);
-  Function::BasicBlockListType &bb_list = 
-    successor->getParent()->getBasicBlockList();
-  
-  for (iterator i = begin(), e = end(); i != e; ++i)
+  Function *F = successor->getParent();
 
+#ifdef LLVM_OLDER_THAN_16_0
+  Function::BasicBlockListType &bb_list =
+    F->getBasicBlockList();
+  for (iterator i = begin(), e = end(); i != e; ++i)
     bb_list.insertAfter(tail->getIterator(), *i);
+#else
+  for (iterator i = begin(), e = end(); i != e; ++i)
+    F->insert(tail->getIterator(), *i);
+#endif
+
   t->setSuccessor(0, entryBB());
 
   t = exitBB()->getTerminator();
@@ -440,11 +446,7 @@ ParallelRegion::Verify()
   return true;
 }
 
-#ifdef LLVM_OLDER_THAN_8_0
-#define PARALLEL_MD_NAME "llvm.mem.parallel_loop_access"
-#else
 #define PARALLEL_MD_NAME "llvm.access.group"
-#endif
 
 /**
  * Adds metadata to all the memory instructions to denote
