@@ -83,14 +83,21 @@ poclu_get_any_device2 (cl_context *context, cl_device_id *device,
 
   err = clGetDeviceIDs (*platform, CL_DEVICE_TYPE_ALL, 1, device, NULL);
   if (err != CL_SUCCESS)
-    return err;
+    {
+
+      err = clGetDeviceIDs (*platform, CL_DEVICE_TYPE_CUSTOM, 1, device, NULL);
+      if (err != CL_SUCCESS)
+        {
+          return err;
+        }
+    }
 
   *context = clCreateContext (NULL, 1, device, NULL, NULL, &err);
   if (err != CL_SUCCESS)
     return err;
 
   *queue = clCreateCommandQueue (*context, *device,
-                                  CL_QUEUE_PROFILING_ENABLE, &err);
+                                 CL_QUEUE_PROFILING_ENABLE, &err);
   if (err != CL_SUCCESS)
     return err;
 
@@ -99,11 +106,13 @@ poclu_get_any_device2 (cl_context *context, cl_device_id *device,
 
 cl_int
 poclu_get_multiple_devices (cl_platform_id *platform, cl_context *context,
+                            cl_char include_custom_dev,
                             cl_uint *num_devices, cl_device_id **devices,
                             cl_command_queue **queues)
 {
   cl_int err;
-  *num_devices = 0;
+  cl_uint num_dev_all = 0;
+  cl_uint num_dev_custom = 0;
   size_t i;
 
   if (context == NULL || devices == NULL || queues == NULL || platform == NULL)
@@ -113,19 +122,57 @@ poclu_get_multiple_devices (cl_platform_id *platform, cl_context *context,
   if (err != CL_SUCCESS)
     return err;
 
-  err = clGetDeviceIDs (*platform, CL_DEVICE_TYPE_ALL, 0, NULL, num_devices);
-  if (err != CL_SUCCESS)
-    return err;
+  err = clGetDeviceIDs (*platform, CL_DEVICE_TYPE_ALL, 0, NULL, &num_dev_all);
+
+  // if custom devices are excluded, continue as normal
+  if (include_custom_dev != 1)
+    {
+      if (err != CL_SUCCESS)
+        return err;
+    }
+  else
+    {
+      cl_int err_custom;
+      err_custom = clGetDeviceIDs (*platform, CL_DEVICE_TYPE_CUSTOM, 0, NULL, &num_dev_custom);
+
+      if (err != CL_SUCCESS && err_custom != CL_SUCCESS)
+        {
+          return err_custom;
+        }
+      if (err != CL_SUCCESS)
+        {
+          num_dev_all = 0;
+        }
+      if (err_custom != CL_SUCCESS)
+        {
+          num_dev_custom = 0;
+        }
+
+    }
+
+  *num_devices = num_dev_all + num_dev_custom;
 
   cl_device_id *devs
-      = (cl_device_id *)calloc (*num_devices, sizeof (cl_device_id));
+      = (cl_device_id *) calloc (*num_devices, sizeof (cl_device_id));
   cl_command_queue *ques
-      = (cl_command_queue *)calloc (*num_devices, sizeof (cl_command_queue));
+      = (cl_command_queue *) calloc (*num_devices, sizeof (cl_command_queue));
 
-  err = clGetDeviceIDs (*platform, CL_DEVICE_TYPE_ALL, *num_devices, devs,
-                        NULL);
-  if (err != CL_SUCCESS)
-    return err;
+  if (num_dev_all != 0)
+    {
+      err = clGetDeviceIDs (*platform, CL_DEVICE_TYPE_ALL, num_dev_all, devs,
+                            NULL);
+      if (err != CL_SUCCESS)
+        return err;
+    }
+
+  if (num_dev_custom != 0)
+    {
+      // add devs to the end of the array
+      err = clGetDeviceIDs (*platform, CL_DEVICE_TYPE_CUSTOM, num_dev_custom, &devs[num_dev_all],
+                            NULL);
+      if (err != CL_SUCCESS)
+        return err;
+    }
 
   *context = clCreateContext (NULL, *num_devices, devs, NULL, NULL, &err);
   if (err != CL_SUCCESS)
@@ -201,6 +248,44 @@ poclu_write_file (const char *filename, char *content, size_t size)
   return 0;
 }
 
+int
+poclu_supports_opencl_30 (cl_device_id *devices, unsigned num_devices)
+{
+  if (num_devices == 0)
+    return 0;
+
+  unsigned supported = 0;
+  char dev_version[256];
+  size_t string_len;
+  for (unsigned i = 0; i < num_devices; ++i)
+    {
+    int err = clGetDeviceInfo (devices[i], CL_DEVICE_VERSION,
+                               sizeof (dev_version), dev_version, &string_len);
+    TEST_ASSERT (err == CL_SUCCESS);
+    if ((string_len >= 15)
+        && (strncmp (dev_version, "OpenCL 3.0 PoCL", 15) == 0))
+        ++supported;
+    }
+  return (supported == num_devices);
+}
+
+int
+poclu_supports_extension (cl_device_id dev, const char *ext)
+{
+  char extensions[8192];
+  int err;
+  size_t real_size = 0;
+  err = clGetDeviceInfo (dev, CL_DEVICE_EXTENSIONS, 0, NULL, &real_size);
+  TEST_ASSERT (err == CL_SUCCESS);
+  TEST_ASSERT (real_size > 0);
+  TEST_ASSERT (real_size < 8192);
+  err = clGetDeviceInfo (dev, CL_DEVICE_EXTENSIONS, real_size, extensions,
+                         NULL);
+  TEST_ASSERT (err == CL_SUCCESS);
+  extensions[real_size] = 0;
+  return (strstr (extensions, ext) != NULL);
+}
+
 cl_int
 poclu_show_program_build_log (cl_program program)
 {
@@ -381,6 +466,14 @@ pocl_getpath (char *path, size_t len, const char *explicit_binary,
   if (access (path, F_OK) == 0)
     return CL_SUCCESS;
 
+  snprintf (path, len, "%s/%s%s", SRCDIR, basename, ext);
+  if (access (path, F_OK) == 0)
+    return CL_SUCCESS;
+
+  snprintf (path, len, "%s/tests/%s%s", SRCDIR, basename, ext);
+  if (access (path, F_OK) == 0)
+    return CL_SUCCESS;
+
   snprintf (path, len, "%s/examples/%s/%s%s", SRCDIR, basename, basename, ext);
   if (access (path, F_OK) == 0)
     return CL_SUCCESS;
