@@ -62,11 +62,7 @@ struct test_struct {
     cl_long elementE;
     cl_float elementF;
     cl_short elementG;
-#ifdef _CL_DISABLE_DOUBLE
     cl_long elementH;
-#else
-    cl_double elementH;
-#endif
 };
 
 static char
@@ -86,11 +82,7 @@ kernelSourceCode[] =
 "    long elementE;\n"
 "    float elementF;\n"
 "    short elementG;\n"
-"#ifdef cl_khr_fp64\n"
-"    double elementH;\n"
-"#else\n"
 "    long elementH;\n"
-"#endif\n"
 "} test_struct;\n"
 "\n"
 "kernel void test_single(int_single input, global int* output) {"
