@@ -1,5 +1,6 @@
 #include <stdio.h>
 #include <stdlib.h>
+#include <string.h>
 
 #include "poclu.h"
 
@@ -49,30 +50,40 @@ main(void)
       TEST_ASSERT(max_mem_alloc_size >= min_max_mem_alloc_size);
 
 #if CL_VERSION_3_0
-      /* OpenCl 3.0 queries */ 
-      cl_device_atomic_capabilities atomic_memory_capability;
-      err = clGetDeviceInfo(devices[j], CL_DEVICE_ATOMIC_MEMORY_CAPABILITIES,
-                            sizeof(cl_device_atomic_capabilities),
-                            &atomic_memory_capability, NULL);
-      CHECK_OPENCL_ERROR_IN("clGetDeviceInfo");
+      char device_version[128];
+      err = clGetDeviceInfo (devices[j], CL_DEVICE_VERSION, 128,
+                             device_version, NULL);
+      CHECK_OPENCL_ERROR_IN ("clGetDeviceInfo");
+      if (strcmp (device_version, "OpenCL 3.0") == 0)
+        {
 
-      cl_device_atomic_capabilities mask = 
-                   (CL_DEVICE_ATOMIC_ORDER_RELAXED
+          /* OpenCl 3.0 queries */
+          cl_device_atomic_capabilities atomic_memory_capability;
+          err = clGetDeviceInfo (devices[j],
+                                 CL_DEVICE_ATOMIC_MEMORY_CAPABILITIES,
+                                 sizeof (cl_device_atomic_capabilities),
+                                 &atomic_memory_capability, NULL);
+          CHECK_OPENCL_ERROR_IN ("clGetDeviceInfo");
+
+          cl_device_atomic_capabilities mask
+              = (CL_DEVICE_ATOMIC_ORDER_RELAXED
+                 | CL_DEVICE_ATOMIC_SCOPE_WORK_GROUP);
+          /* atleast minimum mandated capability is reported */
+          TEST_ASSERT ((mask & atomic_memory_capability) == mask);
+
+          cl_device_atomic_capabilities atomic_fence_capability;
+          err = clGetDeviceInfo (devices[j],
+                                 CL_DEVICE_ATOMIC_FENCE_CAPABILITIES,
+                                 sizeof (cl_device_atomic_capabilities),
+                                 &atomic_fence_capability, NULL);
+          CHECK_OPENCL_ERROR_IN ("clGetDeviceInfo");
+
+          mask = (CL_DEVICE_ATOMIC_ORDER_RELAXED
+                  | CL_DEVICE_ATOMIC_ORDER_ACQ_REL
                   | CL_DEVICE_ATOMIC_SCOPE_WORK_GROUP);
-      /* atleast minimum mandated capability is reported */
-      TEST_ASSERT(( mask & atomic_memory_capability) == mask);
-
-      cl_device_atomic_capabilities atomic_fence_capability;
-      err = clGetDeviceInfo(devices[j], CL_DEVICE_ATOMIC_FENCE_CAPABILITIES,
-                            sizeof(cl_device_atomic_capabilities),
-                            &atomic_fence_capability, NULL);
-      CHECK_OPENCL_ERROR_IN("clGetDeviceInfo");
-
-      mask = (CL_DEVICE_ATOMIC_ORDER_RELAXED 
-            | CL_DEVICE_ATOMIC_ORDER_ACQ_REL
-            | CL_DEVICE_ATOMIC_SCOPE_WORK_GROUP);
-      /* atleast minimum mandated capability is reported */
-      TEST_ASSERT(( mask & atomic_fence_capability) == mask);
+          /* atleast minimum mandated capability is reported */
+          TEST_ASSERT ((mask & atomic_fence_capability) == mask);
+        }
 #endif
 
     }
